import { HotTableProps } from "@handsontable/vue3/types"
import { TricksterInventory } from "./trickster"
import {ColumnInfo, ColumnName, Columns, ColumnSorter, LazyColumn} from "./columns"
import { ColumnSettings } from "handsontable/settings"
import { PredefinedMenuItemKey } from "handsontable/plugins/contextMenu"
import { ref } from "vue"



export interface InventoryTableOptions {
  columns: ColumnSet
  tags: ColumnSet
}

export interface Mappable<T> {
  map<U>(callbackfn: (value: T, index: number, array: T[]) => U, thisArg?: any): U[];
}
export class ColumnSet implements Set<ColumnInfo>, Mappable<ColumnInfo>{
  s: Set<ColumnName> = new Set()
  size: number;
  dirty = ref(0)
  constructor(i?:Iterable<ColumnInfo | ColumnName>){
    if(i){
      for (const a of i) {
        if(Columns[LazyColumn(a)]){
          this.s.add(LazyColumn(a))
        }
      }
    }
    this.size = 0
    this.mark()
  }
  map<U>(callbackfn: (value: ColumnInfo, index: number, array: ColumnInfo[]) => U, thisArg?: any): U[] {
    return Array.from(this.values()).map(callbackfn, thisArg)
  }
  [Symbol.iterator](): IterableIterator<ColumnInfo>{
    return this.values()
  }
  [Symbol.toStringTag] = "ColumnSet";
  entries(): IterableIterator<[ColumnInfo, ColumnInfo]>{
    return Array.from(this.values()).map((x):[ColumnInfo,ColumnInfo]=>{return [x,x]}).values()
  }
  keys(): IterableIterator<ColumnInfo>{
    return this.values()
  }
  forEach(callbackfn: (value: ColumnInfo, value2: ColumnInfo, set: Set<ColumnInfo>) => void, thisArg?: any): void{
    Array.from(this.values()).forEach((v)=>{
      if(this.has(v)) {
        callbackfn(v, v, new Set(this.values()))
      }
    }, thisArg)
  }
  values(): IterableIterator<ColumnInfo>{
    return Array.from(this.s.values()).sort(ColumnSorter).map((a, b)=>{
      return Columns[a]
    }).values()
  }
  mark() {
    this.dirty.value = this.dirty.value + 1
    this.size = this.s.size
  }
  add(value: ColumnInfo | ColumnName): this {
    this.mark()
    this.s.add(LazyColumn(value))
    return this
  }
  clear(): void {
    this.mark()
    this.s.clear()
  }
  delete(value: ColumnInfo | ColumnName): boolean{
    this.mark()
    return this.s.delete(LazyColumn(value))
  }
  has(value: ColumnInfo | ColumnName): boolean{
    return this.s.has(LazyColumn(value))
  }
}

export class InventoryTable {
  inv!:TricksterInventory
  o!: InventoryTableOptions

  constructor(inv:TricksterInventory, o:InventoryTableOptions) {
    this.setInv(inv)
    this.setOptions(o)
  }
  setOptions(o:InventoryTableOptions) {
    this.o = o
  }
  setInv(inv:TricksterInventory) {
    this.inv = inv
  }
  getTableColumnNames(): string[] {
    return this.o.columns.map(x=>x.displayName)
  }
  getTableColumnSettings(): ColumnSettings[] {
    return this.o.columns.map(x=>({
      renderer: x.renderer ? x.renderer : "text",
      dropdownMenu: x.filtering ? DefaultDropdownItems() : false,
      readOnly: x.writable ? false : true,
      selectionMode: x.writable ? "multiple" : "single"
    }))
  }
  getTableRows():any[][] {
    return Object.values(this.inv.items)
    .filter((item):boolean=>{
      let found = false
      if(this.o.tags.has("All")) {
        found = true
      }
      if(this.o.tags.s.size > 0) {
        for(const tag of this.o.tags.values()) {
          if(tag.name =="All") {
            continue
          }
          let chk = tag.getter(item)
          if(this.o.tags.has("All")) {
            if(chk === 1) {
              found = false
              break
            }
          }else {
            if(chk === 1) {
              found = true
              break
            }
          }
        }
      }else {
        found = true
      }
      return found
    })
    .map((item)=>{
      return this.o.columns.map(x=>{
        return x.getter(item)
      })
    })
  }
  BuildTable():TableRecipe {
    const s = DefaultSettings()
    const dat = this.getTableRows()
    return {
      data: dat,
      settings: {
        data: dat,
        colHeaders:this.getTableColumnNames(),
        columns:this.getTableColumnSettings(),
        ...s
      },
    }
  }
}

export interface TableRecipe {
  data: any[][]
  settings: HotTableProps
}

export const DefaultDropdownItems = ():PredefinedMenuItemKey[]=>['filter_by_condition' , 'filter_operators' ,'filter_by_condition2' , 'filter_by_value' , 'filter_action_bar']
export const DefaultSettings = ():HotTableProps=>{
  return {
    trimDropdown: true,
    filters: true,
    manualRowMove: false,
    manualColumnMove: false,
    allowInsertRow: false,
    allowInsertColumn: false,
    allowRemoveRow: false,
    allowRemoveColumn: false,
    allowHtml: true,
    disableVisualSelection: false,
    columnSorting: {
      indicator: true,
      headerAction: true,
    },
    className: 'htLeft',
    contextMenu: false,
    dropdownMenu: false,
    readOnlyCellClassName: "",
    licenseKey:"non-commercial-and-evaluation",
  }
}
